@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-from Acquisition import aq_inner
+from Acquisition import aq_base
 from Products.CMFCore.interfaces import ISiteRoot
 from Products.CMFCore.utils import getToolByName
 from datetime import datetime
@@ -11,10 +11,13 @@ from zope.component.hooks import getSite
 from zope.i18n import translate
 from zope.i18nmessageid import MessageFactory
 from zope.schema.interfaces import IVocabularyFactory
-from z3c.form.interfaces import IAddForm
+from z3c.form.interfaces import IForm
+from zope.component import getMultiAdapter
+from zope.component import ComponentLookupError
+import json
+from zope.globalrequest import getRequest
 
-_ = MessageFactory('plone.app.widgets')
-_plone = MessageFactory('plone')
+_ = MessageFactory('plone')
 
 
 try:
@@ -64,9 +67,9 @@ def get_date_options(request):
             'format': translate(
                 _('pickadate_date_format', default='mmmm d, yyyy'),
                 context=request),
-            'placeholder': translate(_plone('Enter date...'), context=request),
-            'today': translate(_plone(u"Today"), context=request),
-            'clear': translate(_plone(u"Clear"), context=request),
+            'placeholder': translate(_('Enter date...'), context=request),
+            'today': translate(_(u"Today"), context=request),
+            'clear': translate(_(u"Clear"), context=request),
         }
     }
 
@@ -77,8 +80,8 @@ def get_datetime_options(request):
         'format': translate(
             _('pickadate_time_format', default='h:i a'),
             context=request),
-        'placeholder': translate(_plone('Enter time...'), context=request),
-        'today': translate(_plone(u"Today"), context=request),
+        'placeholder': translate(_('Enter time...'), context=request),
+        'today': translate(_(u"Today"), context=request),
     }
     return options
 
@@ -118,16 +121,33 @@ def get_ajaxselect_options(context, value, separator, vocabulary_name,
 def get_relateditems_options(context, value, separator, vocabulary_name,
                              vocabulary_view, field_name=None):
     portal = get_portal()
-    options = get_ajaxselect_options(portal, value, separator,
+    options = get_ajaxselect_options(context, value, separator,
                                      vocabulary_name, vocabulary_view,
                                      field_name)
-
+    if IForm.providedBy(context):
+        context = context.context
+    request = getRequest()
+    msgstr = translate(_(u'Search'), context=request)
+    options.setdefault('searchText', msgstr)
+    msgstr = translate(_(u'Entire site'), context=request)
+    options.setdefault('searchAllText', msgstr)
+    msgstr = translate(_('tabs_home',
+                       default=u'Home'),
+                       context=request)
+    options.setdefault('homeText', msgstr)
     options.setdefault('folderTypes', ['Folder'])
-    properties = getToolByName(context, 'portal_properties')
-    if properties:
-        options['folderTypes'] = properties.site_properties.getProperty(
-            'typesLinkToFolderContentsInFC', options['folderTypes'])
+    options.setdefault(
+        'treeVocabularyUrl',
+        '{}/@@getVocabulary?name=plone.app.vocabularies.Catalog'.format(
+            portal is not None and portal.absolute_url() or '')
+    )
+    options.setdefault('sort_on', 'sortable_title')
+    options.setdefault('sort_order', 'ascending')
 
+    nav_root = getNavigationRootObject(context, get_portal())
+    options['rootPath'] = (
+        '/'.join(nav_root.getPhysicalPath()) if nav_root else '/'
+    )
     return options
 
 
@@ -145,36 +165,19 @@ def get_querystring_options(context, querystring_view):
 
 
 def get_tinymce_options(context, field, request):
-    args = {'pattern_options': {}}
-    utility = getToolByName(aq_inner(context), 'portal_tinymce', None)
-    if not utility:
-        return args
-    config = utility.getConfiguration(context=context,
-                                      field=field,
-                                      request=request)
-
-    config['content_css'] = config['portal_url'] + '/base.css'
-    del config['customplugins']
-    del config['plugins']
-    del config['theme']
-
-    args['pattern_options'] = {
-        'relatedItems': {
-            'vocabularyUrl': config['portal_url'] +
-            '/@@getVocabulary?name=plone.app.vocabularies.Catalog'
-        },
-        'rel_upload_path': '@@fileUpload',
-        'folder_url': config['document_base_url'],
-        'tiny': config,
-        'prependToUrl': 'resolveuid/',
-        'linkAttribute': 'UID',
-        'prependToScalePart': '/@@images/image/',
-        'folderTypes': utility.containsobjects.replace('\n', ','),
-        'imageTypes': utility.imageobjects.replace('\n', ','),
-        'anchorSelector': utility.anchor_selector,
-        'linkableTypes': utility.linkable.replace('\n', ',')
-    }
-    return args
+    """
+    We're just going to be looking up settings from
+    plone pattern options
+    """
+    options = {}
+    try:
+        pattern_options = getMultiAdapter(
+            (context, request, field),
+            name="plone_settings").tinymce()['data-pat-tinymce']
+        options = json.loads(pattern_options)
+    except (ComponentLookupError, AttributeError):
+        pass
+    return options
 
 
 def get_portal():
@@ -200,7 +203,7 @@ def get_portal_url(context):
 
 
 def get_context_url(context):
-    if IAddForm.providedBy(context):
+    if IForm.providedBy(context):
         # Use the request URL if we are looking at an addform
         url = context.request.get('URL')
     elif hasattr(context, 'absolute_url'):
@@ -210,3 +213,10 @@ def get_context_url(context):
     else:
         url = get_portal_url(context)
     return url
+
+
+def get_widget_form(widget):
+    form = getattr(widget, 'form', None)
+    if getattr(aq_base(form), 'parentForm', None) is not None:
+        form = form.parentForm
+    return form
